
local PlayerGUID
local spaced  -- tracked to avoid adding an empty line (" ") when unnecessary

local function isLastLineEmpty(tooltip)
  local n, i = tooltip:GetName(), tooltip:NumLines()
  local left, right = n.."TextLeft"..i, n.."TextRight"..i
  left, right = getglobal(left), getglobal(right)
  left, right = left:GetText() or "", right:GetText() or ""
  left, right = strtrim(left), strtrim(right)
  return (left == "" and right == "");
end

local function addline(tooltip, label, value, labelR, valueR)
  local textL = value and "|cff7eff00"..label..":|r "..value or " "
  if (not spaced) then
    if (not isLastLineEmpty(tooltip)) then  tooltip:AddLine(" ");  end
    spaced = true
  end
  if (labelR) then
    tooltip:AddDoubleLine(textL, "|cff7eff00"..labelR..":|r "..valueR, 0.741, 1, 0.467,0.741, 1, 0.467)
  else
    tooltip:AddLine(textL, 0.741, 1, 0.467)
  end
end

local function AppendProgressToTooltip(tooltip, id, GUID)
  -- Compensating for what seems like a bug that causes certain achievements, such as "It's Happy Hour Somewhere,"
  -- to show their criteria as incomplete even when the achievement has been earned. Hopefully this will be
  -- unnecessary in the near future. (Also see "override_completed" below.)
  local _, _, _, override_completed = GetAchievementInfo(id)

  local text = (PlayerGUID == GUID) and OVERACHIEVER_STRINGS_PROGRESS or OVERACHIEVER_STRINGS_YOURPROGRESS;
  local prev, prevcompleted
  local criteriaString, _, completed, quantity, totalQuantity, name, quantityString
  for i=1,GetAchievementNumCriteria(id) do
    criteriaString, _, completed, quantity, totalQuantity, name, _, _, quantityString, _ = GetAchievementCriteriaInfo(id, i);
    completed = completed or override_completed
    if (strfind(quantityString, "/")) then
      if (prev) then
        addline(tooltip, text, prev);
        if (prevcompleted) then  tooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready");  end
      end
      prev, prevcompleted = quantityString, completed
    end
  end
  return text, prev, prevcompleted
end

local function ExamineAchievementTip(tooltip, link)
  if ( tooltip:NumLines() > 0 and
      (Overachiever_Settings.Tooltip_ShowProgress or Overachiever_Settings.Tooltip_ShowID) ) then
    local _, _, id = strfind(link, "achievement:(.+):")
    if (id) then
      PlayerGUID = PlayerGUID or strsub(UnitGUID("player"), 3)
      local GUID
      id, GUID = strsplit(":", id);
      id = tonumber(id)
      if (id) then
        spaced = nil
        local progresstext, progressval
        if ( Overachiever_Settings.Tooltip_ShowProgress and
            (PlayerGUID == GUID or Overachiever_Settings.Tooltip_ShowProgress_Other) ) then
          progresstext, progressval, progresscompleted = AppendProgressToTooltip(tooltip, id, GUID)
        end
        if (Overachiever_Settings.Tooltip_ShowID) then
          addline(tooltip, progresstext, progressval, "ID", id)
          if (progresscompleted and progressval) then
            tooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready");
          end
        elseif (progressval) then
          addline(tooltip, progresstext, progressval);
          if (progresscompleted) then
            tooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready");
          end
        end
        tooltip:Show()
      end
    end
  end
end

hooksecurefunc(ItemRefTooltip, "SetHyperlink", ExamineAchievementTip)
hooksecurefunc(GameTooltip, "SetHyperlink", ExamineAchievementTip)
